#! perl -w

# Copyright (c) 1991-2001 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# rtcomp.pl - Compilation controller

# Force autoflush for nice error messages

select STDERR; $| = 1;
select STDOUT; $| = 1;

# Get RTS_HOME and load in rterror

$RTS_HOME = $0;
$RTS_HOME =~ s{\\}{/}g;
$RTS_HOME =~ s{/codegen/rtcomp.pl$}{};

push @INC, $RTS_HOME;

# load in the error handler

require 'codegen/rterror.pl';

# Silence perl: other uses are in rterror.pl.
$rterror::maxError = $rterror::maxError;

# Get the arguments

$component = 0;
@deppath   = ();
$spacify   = '';
$srcfile   = '';
$srcobject = '';
$vendor    = 'generic';

while( $#ARGV >= 0 )
{
    $arg = shift @ARGV;
    if   ( $arg eq '-vendor'        ) { $vendor                 = shift @ARGV; }
    elsif( $arg eq '-spacify'       ) { $spacify                = shift @ARGV; }
    elsif( $arg eq '-component'     ) { $component              = 1;           }
    elsif( $arg eq '-componentname' ) { $rterror::componentName = shift @ARGV; }
    elsif( $arg eq '-I'             ) { push @deppath,            shift @ARGV; }
    elsif( $arg eq '-src'           ) { $srcobject              = shift @ARGV;
					$srcfile                = shift @ARGV; }
    elsif( $arg eq '--'             ) { last; }
    else
    {
	&rterror::print_error( '', "Unknown rtcomp.pl option $arg", 1 );
    }
}

unless( $srcfile )
{
    &rterror::print_error( '', 'rtcomp.pl: missing -src option', 1 );
    exit 1;
}

if( $component and $rterror::componentName eq '' )
{
    $rterror::componentName = $srcobject;
}

foreach $loc ( @deppath )
{
    $loc =~ s{\\}{/}g;
}

# pick out the src directory

$srcdir = $srcfile;
$srcdir =~ s{(.*)[/\\][^/\\]*}{$1};

# load in configs (for now, just the compiler)

require "codegen/compiler/$vendor/comp.pl";

print STDERR '!> Compiling ', $srcobject, "\n";

foreach $arg ( @ARGV )
{
    $arg =~ s{^.*\s.*$}{"$&"};
}
$command = join( ' ', @ARGV );

$OS          = '?' unless $OS          = $ENV{'OS'};
$ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};

$errRedirect = ( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
			? ''
			: '2>&1';

open ERRORS, "$command $errRedirect |";

@errors = <ERRORS>;

close ERRORS;
$retcode = $?;

print @errors;
&rterror::handle_errors();

####
#### MAKEDEPEND
####

sub find_file($)
{
    my( $file ) = @_;
    my( $path );

    # a special case: has a complete path already been provided?
    return $file if -e "$file";

    foreach $loc ( @deppath )
    {
	$path = "$loc/$file";
	return $path if -e $path;
    }

    return '';
}

# srcfile is first, let's make all the slashes go one way
$srcfile =~ s{\\}{/}g;
@deplist = ( $srcfile );
$deps{$srcfile} = 1;

foreach $file ( @deplist )
{
    unless( open FILE, $file )
    {
	&rterror::print_error( '', "Could not open [$file]", 1 );
	exit 1;
    }

    while( <FILE> )
    {
	# recognize: include <X> or include "X"
	if( m{^\s*#\s*include\s*\<([^\>]*)\>} or
	    m{^\s*#\s*include\s*\"([^\"]*)\"} )
	{
	    my( $depfile ) = &find_file( $1 );

	    if( $depfile && ! $deps{$depfile} )
	    {
		$deps{$depfile} = 1;
		push @deplist, $depfile;
	    }
	}
    }

    close FILE;
}

if( $spacify eq 'dq' )
{
    foreach $dep ( @deplist )
    {
	$dep =~ s{^.*\s.*$}{"$&"};
    }
}
elsif( $spacify eq 'bs' )
{
    foreach $dep ( @deplist )
    {
	$dep =~ s{\s}{\\$&}g;
    }
}

$depfile = $component ? "RTSystem/$srcobject.dep"
		      :          "$srcobject.dep";

unless( open DEPFILE, ">$depfile" )
{
    &rterror::print_error( '', "Cannot write to '$depfile'", 0 );
    exit 1;
}

binmode( DEPFILE );

print DEPFILE $srcobject, ( $component ? "_SYS" : "_" ), "DEPS =";

foreach $dep ( @deplist )
{
    print DEPFILE " \\\n\t", $dep;
}

print DEPFILE "\n";

close DEPFILE;

if( $rterror::worstError < $rterror::maxError )
{
    if( $rterror::worstError == 0 )
    {
	exit 1;
    }
}
elsif( $retcode & 0xff )
{
    &rterror::print_error( '',
			   "command [$command] caught signal [$retcode]",
			   0 );
    exit 1;
}
elsif( $retcode )
{
    $retcode >>= 8;
    &rterror::print_error( '',
			   "command [$command] returned error [$retcode]",
			   0 );
    exit 1;
}
