#! perl -w

# Copyright (c) 1991-2001 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# rtdep.pl - Dependency finder for header files only (for model interface)

# Force autoflush for nice error messages

select STDERR; $| = 1;
select STDOUT; $| = 1;

# Get RTS_HOME and load in rterror

$rtsHome = $0;
$rtsHome =~ s:\\:/:g;
$rtsHome =~ s:/codegen/rtdeps.pl$::;

# load in the error handler

require "$rtsHome/codegen/rterror.pl";

# Silence perl: other uses are in rterror.pl.
$rterror::maxError = $rterror::maxError;

# Get the arguments

while( $#ARGV >= 0 )
{
    $arg = shift @ARGV;
    if   ( $arg eq '-I'   ) { push @deppath,   shift @ARGV; }
    elsif( $arg eq '-src' ) { $srcobject     = shift @ARGV;
			      $srcfile       = shift @ARGV; }
    else
    {
	&rterror::print_error( '', "Unknown rtcomp.pl option $arg", 1 );
    }
}

####
#### MAKEDEPEND
####

sub find_file($)
{
    my( $file ) = @_;

    if( -e $file )
    {
	# a special case: has a complete path already been provided?
	return $file;
    }

    foreach $loc ( @deppath )
    {
	if( -e "$loc/$file" )
	{
	    return "$loc/$file";
	}
    }

    return '';
}

# srcfile is first, let's make all the slashes go one way
$srcfile =~ s:\\:/:g;
@deplist = ( $srcfile );
$deps{$srcfile} = 1;

foreach $file ( @deplist )
{
    if( ! open FILE, "$file" )
    {
	&rterror::print_error( '', "Could not open [$file]", 1 );
	exit 1;
    }

    while( <FILE> )
    {
	# recognize: include <X> or include "X"
	if( m/^\s*#\s*include\s*\<([^\>]*)\>.*$/o or
	    m/^\s*#\s*include\s*\"([^\"]*)\".*$/o )
	{
	    my( $depfile ) = find_file( $1 );

	    if( $depfile && ! $deps{$depfile} )
	    {
		$deps{$depfile} = 1;
		push @deplist, $depfile;
	    }
	}
    }

    close FILE;
}

$depfile = "$srcobject.dep";

if( ! open DEPFILE, ">$depfile" )
{
    &rterror::print_error( '', "Cannot write to '$depfile'", 0 );
    exit 1;
}

binmode( DEPFILE );

print DEPFILE $srcobject, "_DEPS =";

foreach $dep ( @deplist )
{
    if( $dep =~ /\s/ )
    {
	$dep = '"' . $dep . '"';
    }

    print DEPFILE " \\\n\t$dep";
}

print DEPFILE "\n";

close DEPFILE;

if( $rterror::worstError < $rterror::maxError )
{
    if( $rterror::worstError == 0 )
    {
	exit 1;
    }
}
