#! perl -w

# Copyright (c) 1991-2001 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

package rterror;

# Tags look like this

$startpattern   = '// {{{RME (.*)$';
$endpattern     = '// }}}RME';
$ignorepattern1 = '// {{{USR';
$ignorepattern2 = '// }}}USR';

# print_error - Subroutine to print out an error
# Arguments
#   <object>    object of error
#   <message>   string containing the message
#   <level>     level giving the severity
#       0 = error
#       1 = warning

$maxError      = 2;
$worstError    = $maxError;
$componentName = '';

@levelName = ( 'error', 'warning' );

sub print_error($$$)
{
    my( $object, $msg, $level ) = @_;

    print 'GES';
    print ' ', $object if $object;

    $msg =~ s/'/''/g;

    print " description '", $msg,
	  "' severity '",   $levelName[ $level ],
	  "'\n";

    $worstError = $level if $worstError > $level;
}

sub handle_errors()
{
    my $error;

    error_loop: foreach $error ( @main::errors )
    {
	chomp $error;

	foreach $handler ( @config::handlers )
	{
	    ( $pattern, $action ) = @$handler;

	    if( $error =~ /$pattern/ )
	    {
		&$action;
		next error_loop;
	    }
	}

	# not handled
	# &print_error( '', $error, 1 );
    }
}

# Handlers

sub action_ignore
{
}

sub action_print($$$$)
{
    my( $file, $lineno, $msg, $level ) = @_;

    if( open FILE, $file )
    {
	# found file directly
    }
    elsif( $main::srcdir && open FILE, "$main::srcdir/$file" )
    {
	# found file in source directory
    }
    else
    {
	&print_error( '', "Unable to open file: $file", 0 );
	return;
    }

    my $object     = '';
    my $gesmsg     = '';
    my @taglineno  = ();
    my @taginfo    = ();
    my $filelineno = 0;
    my $fileline;

    while( $fileline = <FILE> )
    {
	++$filelineno;
	chomp $fileline;

	if( $filelineno == $lineno )
	{
	    if( $#taglineno >= 0 )
	    {
		$lineno -= $taglineno[$#taglineno];

		my %tags = ();
		my $tag;

		foreach $tag ( @taginfo )
		{
		    my $tagtext = $tag;

		    while( $tagtext =~ s/^\s*(\w+)\s+'([^']+)'\s*// )
		    {
			my( $key, $value ) = ( $1, $2 );

			my $prefix = $tags{$key} || '';

			if( $prefix && $key eq 'classifier' && $value !~ m{::} )
			{
			    $tags{$key} = $prefix . '::' . $value;
			}
			else
			{
			    $tags{$key} = $value;
			}
		    }
		}

		$tags{'line'} = $lineno;

		my @list = ();

		foreach $key ( sort keys %tags )
		{
		    push @list, $key, "'$tags{$key}'";
		}

		$object = join( ' ', @list );
		$gesmsg = $msg;
	    }
	    last;
	}
	elsif( $fileline =~ m{$startpattern}o )
	{
	    push @taginfo,   $1;
	    push @taglineno, $filelineno;
	}
	elsif( $fileline =~ m{$endpattern}o )
	{
	    pop @taglineno;
	    pop @taginfo;
	}
	elsif( $fileline =~ m{$ignorepattern1}o
	    or $fileline =~ m{$ignorepattern2}o )
	{
	    ++$taglineno[$#taglineno] if $#taglineno >= 0;
	}
	elsif( $fileline =~ m{^\{$} ) # }
	{
	    $taglineno[$#taglineno] = $filelineno if $#taglineno >= 0;
	}
    }

    close FILE;

    $gesmsg = "$file:$lineno - $msg" unless $gesmsg;

    &print_error( $object, $gesmsg, $level );
}

sub action_message($$)
{
    my( $msg, $level ) = @_;

    if( $componentName ne '' )
    {
	&print_error( "component '$componentName'", $msg, $level );
    }
    else
    {
	&print_error( '', $msg, $level );
    }
}

return 1;
