#! perl -w

# Copyright (c) 1991-2001 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# rtlink.pl - Link controller

# Unbuffer stdout

select STDERR; $| = 1;
select STDOUT; $| = 1;

# Get RTS_HOME and load in rterror

$rtsHome = $0;
$rtsHome =~ s:\\:/:g;
$rtsHome =~ s:/codegen/rtlink.pl$::;

# load in the error handler

require "$rtsHome/codegen/rterror.pl";

# Silence perl: other uses are in rterror.pl.
$rterror::componentName = $rterror::componentName;
$rterror::maxError      = $rterror::maxError;

# Get the arguments

$isLib      = 0;
$isPurify   = 0;
$targetname = '';

while( $#ARGV >= 0 )
{
    $arg = shift @ARGV;
    if   ( $arg eq '-vendor' ) { $vendor     = shift @ARGV; }
    elsif( $arg eq '-purify' ) { $isPurify   = 1; }
    elsif( $arg eq '-exe'    ) { $targetname = shift @ARGV; $isLib = 0; }
    elsif( $arg eq '-lib'    ) { $targetname = shift @ARGV; $isLib = 1; }
    elsif( $arg eq '-componentname' ) { $rterror::componentName = shift @ARGV; }
    elsif( $arg eq '--'      ) { last; }
    else
    {
	&rterror::print_error( '', "Unknown rtlink.pl option $arg", 1 );
    }
}

%config = (
	    'exename'       => '',
	    'purifyexename' => ''
	  );

# Read the executable name from the rtbuild.dat file.

if( open DAT, '../rtbuild.dat' )
{
    while( <DAT> )
    {
	chomp;
	( $key, $value ) = split( '=' );
	$config{$key} = $value;
    }
    close DAT;
}

if( ( $isPurify ? $config{purifyexename}
		: $config{exename}       ) ne $targetname )
{
    if( $targetname =~ /^\// || $targetname =~ /^\\/ || $targetname =~ /^.:/ )
    {
	$exepath = $targetname;
    }
    else
    {
	$exepath = 'build/' . $targetname;
    }

    if( $isPurify )
    {
	$config{purifyexename} = $exepath;
    }
    else
    {
	$config{exename} = $exepath;
    }

    if( open DAT, '>../rtbuild.dat' )
    {
	foreach $key ( keys %config )
	{
	    print DAT $key, '=', $config{$key}, "\n";
	}
	close DAT;
    }
}

# load in configs (for now, just the compiler)

require "$rtsHome/codegen/compiler/$vendor/link.pl";

if( $isLib )
{
    print STDERR "!> Building library $targetname\n";
}
else
{
    print STDERR "!> Linking $targetname\n";
}

$command = '';
foreach $arg ( @ARGV )
{
    if( $arg =~ / / )
    {
	$arg = '"' . $arg . '"';
    }
    if( $command )
    {
	$command .= ' ';
    }
    $command .= $arg;
}

$OS          = '?' unless $OS          = $ENV{'OS'};
$ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};

$errRedirect = ( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
			? ''
			: '2>&1';

open OPEN, "$command $errRedirect |";

@errors = <OPEN>;

close OPEN;
$retcode = $?;

print @errors;
&rterror::handle_errors();

# now handle return code from command

if( $rterror::worstError < $rterror::maxError )
{
    if( $rterror::worstError == 0 )
    {
	exit 1;
    }
}
elsif( $retcode & 0xff )
{
    &rterror::print_error( '',
			   "command [$command] caught signal [$retcode]",
			   0 );
    exit 1;
}
elsif( $retcode )
{
    $retcode >>= 8;
    &rterror::print_error( '',
			   "command [$command] returned error [$retcode]",
			   0 );
    exit 1;
}
