/*
 * Copyright (c) 1991-2001 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>

#include <stdio.h>
#include <sched.h>

int main( int argc, const char * const * argv )
{
    int policy   = -1;
    int priority = 0;

    for( int i = 0; ++i < argc; )
    {
	const char * option = argv[ i ];

	if( RTMemoryUtil::strcmp( option, "-SCHED_", 7 ) != 0 )
	    continue;

	option += 7;

	if(      RTMemoryUtil::strcmp( option, "FIFO",  4 ) == 0 )
	    option += 4, policy = SCHED_FIFO;
	else if( RTMemoryUtil::strcmp( option, "RR",    2 ) == 0 )
	    option += 2, policy = SCHED_RR;
	else if( RTMemoryUtil::strcmp( option, "OTHER", 5 ) == 0 )
	    option += 5, policy = SCHED_OTHER;
	else
	    continue;

	if( *option == '\0' )
	    priority = 0;
	else if( *option == '=' )
	    priority = RTMemoryUtil::atoi( &option[ 1 ] );
	else
	    continue;
    }

    while( policy != -1 )
    {
	int         priority_min;
	int         priority_max;
	sched_param sched;

	if( sched_getparam( 0, &sched ) != 0 )
	    perror( "sched_getparam" );

	else if( (priority_min = sched_get_priority_min( policy )) == -1 )
	    perror( "sched_get_priority_min" );

	else if( (priority_max = sched_get_priority_max( policy )) == -1 )
	    perror( "sched_get_priority_max" );

	else if( priority < priority_min || priority > priority_max )
	    RTDiag::out()
		<< "Priority (" << priority << ") out of range: ["
		<< priority_min << ".." << priority_max << "]\n";

	else if( sched.sched_priority = priority,
		 sched_setscheduler( 0, policy, &sched ) != 0 )
	    perror( "sched_setscheduler" );

	else
	    break;

	return 1;
    }

    return RTMain::entryPoint( argc, argv );
}
