/*
 * Copyright (c) 1991-2001 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

#include <unistd.h>
#include <sys/types.h>

static void * thread_run( void * job )
{
    ((RTJob *)job)->mainLoop();

    return (void *)0;
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job ( job )
    , _info( new RTThreadInfo )
{
    const char     * failure;
    pthread_attr_t   attr;
    sched_param      sched;
    int              policy;
    int              priority_min;
    int              priority_max;
    int              min_stack_size = 16384;

    for(;;)
    {
	if( (policy = sched_getscheduler( 0 )) == -1 )
	{
	    failure = "sched_getscheduler";
	    break;
	}

	if( (priority_min = sched_get_priority_min( policy )) == -1 )
	{
	    failure = "sched_get_priority_min";
	    break;
	}

	if( prio < priority_min )
	    prio = priority_min;

	if( (priority_max = sched_get_priority_max( policy )) == -1 )
	{
	    failure = "sched_get_priority_max";
	    break;
	}

	if( prio > priority_max )
	    prio = priority_max;

	if( sched_getparam( 0, &sched ) != 0 )
	{
	    failure = "sched_getparam";
	    break;
	}

	if( pthread_attr_init( &attr ) != 0 )
	{
	    failure = "pthread_attr_init";
	    break;
	}

	if( pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE ) != 0 )
	{
	    failure = "pthread_attr_setdetachstate";
	    break;
	}

	if( pthread_attr_setscope( &attr, PTHREAD_SCOPE_SYSTEM ) != 0 )
	{
	    failure = "pthread_attr_setscope";
	    break;
	}

	if( stack_size < min_stack_size )
	    stack_size = min_stack_size;

	if( pthread_attr_setstacksize( &attr, stack_size ) != 0 )
	{
	    failure = "pthread_attr_setstacksize";
	    break;
	}

	if( pthread_attr_setinheritsched( &attr, PTHREAD_EXPLICIT_SCHED ) != 0 )
	{
	    failure = "pthread_attr_setinheritsched";
	    break;
	}

	if( pthread_attr_setschedpolicy( &attr, policy ) != 0 )
	{
	    failure = "pthread_attr_setschedpolicy";
	    break;
	}

	sched.sched_priority = prio;

	if( pthread_attr_setschedparam( &attr, &sched ) != 0 )
	{
	    failure = "pthread_attr_setschedparam";
	    break;
	}

	if( pthread_create( &_info->tid, &attr, thread_run, job ) != 0 )
	{
	    failure = "pthread_create";
	    break;
	}

	_info->valid = 1;

	if( pthread_attr_destroy( &attr ) != 0 )
	{
	    failure = "pthread_attr_destroy";
	    break;
	}

	return;
    }

    RTDiag::panic( failure );
}

RTThreadInfo::RTThreadInfo( void )
    : valid( 0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( valid == 0 )
	return; // never successfully created

    void * status = (void *)0;

    if( pthread_join( tid, &status ) != 0 )
	RTDiag::panic( "pthread_join" );
}
