#! perl -w

# Copyright (c) 1991-2001 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

sub mySystem
{
    my( $OS, $ROSERT_HOST, $rc );

    $OS          = '?' unless $OS          = $ENV{'OS'};
    $ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};

    if( $debug ) { print "+ ", join( ' ', @_ ), "\n"; }

    if( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
    {
	$rc = system( join( ' ', @_ ) );
    }
    else
    {
	$rc = system( @_ );
    }

    if( $rc != 0 )
    {
	exit $rc if( $rc & 0xFF );
	exit $rc >> 8;
    }
}

sub fatal
{
    print STDERR "$_[0]\n";
    exit 1;
}

# Option   Type     Default   Description
# ======   ====     =======   ===========
# -add     list     ar,r      Command to add to an archive
# -create  list     ar,rc     Command to create an archive
# -debug   boolean  false     Debugging this script?
# -input   boolean  false     Output library is also an input to 'add' command?
# -limit   numeric  no limit  Limits command length
# -output  string   ''        Prefix for output archive
# -prefix  string   ''        Prefix for object files
# -ranlib  string   nothing   Command to update symbol table
# -suffix  string   .o        Suffix added to files listed in %file

$suffix = '.o';
@add_cmd    = ( 'ar', 'r'  );
@create_cmd = ( 'ar', 'rc' );
$debug      = 0;
$input      = 0;
$limit      = undef;
$prefix     = '';
$output     = '';
@ranlib_cmd = ();

while( $#ARGV >= 0 )
{
    $arg = $ARGV[0];
    if   ( $arg =~ m{^-add=(.*)$}    ) { @add_cmd    = split( ',', $1 ); }
    elsif( $arg =~ m{^-create=(.*)$} ) { @create_cmd = split( ',', $1 ); }
    elsif( $arg =~ m{^-debug$}       ) { $debug      =  1; }
    elsif( $arg =~ m{^-input$}       ) { $input      =  1; }
    elsif( $arg =~ m{^-limit=(\d*)$} ) { $limit      = $1; }
    elsif( $arg =~ m{^-output=(.*)$} ) { $output     = $1; }
    elsif( $arg =~ m{^-prefix=(.*)$} ) { $prefix     = $1; }
    elsif( $arg =~ m{^-ranlib=(.*)$} ) { @ranlib_cmd = split( ',', $1 ); }
    elsif( $arg =~ m{^-suffix=(.*)$} ) { $suffix     = $1; }
    elsif( $arg !~ m{^-}             ) { last; }
    else
    {
	&fatal( "Unknown option: '$arg'.\n" );
    }
    shift;
}

$#ARGV >= 1 || &fatal( "Usage: ${0} ar [options] archive objects ..." );

$archive = $ARGV[0];
shift;

@cmd = ( @create_cmd, "$output$archive" );
$cmd_len = length( join( ' ', @cmd ) );
$object_count = 0;

sub add_object
{
    my( $object )     = "$prefix$_[0]";
    my( $object_len ) = length( $object ) + 1;

    if( $object_count > 0 && defined( $limit ) )
    {
	if( $cmd_len + $object_len >= $limit )
	{
	    &mySystem( @cmd );

	    @cmd = ( @add_cmd, "$output$archive" );
	    push( @cmd, $archive ) if( $input );
	    $cmd_len = length( join( ' ', @cmd ) );
	    $object_count = 0;
	}
    }
    push( @cmd, $object );
    $cmd_len += $object_len;
    $object_count += 1;
}

if( $debug ) { print "+ unlink( $archive )\n"; }

unlink( $archive );

foreach $arg ( @ARGV )
{
    if( $arg =~ m{^%(.*)$} )
    {
	$list = $1;
	if( ! open( LIST, "<$1" ) )
	{
	    &fatal( "Cannot open '$list'\n" );
	}

	while( <LIST> )
	{
	    chomp;
	    &add_object( "$_$suffix" );
	}

	close( LIST );
    }
    else
    {
	&add_object( $arg );
    }
}

&mySystem( @cmd );

&mySystem( @ranlib_cmd, $archive ) if @ranlib_cmd;
