#! perl -w

# Copyright (c) 1991-2001 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# Tools to extract the value of a key from the registry.
#
# The registry value corresponding to key\subkey is printed to stdout.
# If the registry value cannot be read for any reason, default will be
# output instead.

# -----------------------------------------------------------------------------
sub tempfile
{
    # Create a tempfile of the form %TMP%\ot<pid>.<NNN>.tmp,
    # where NNN makes the filename unique
    # This is obviously not very efficient...

    my( $prefix, $i, $filename );

    $prefix  = $ENV{'TEMP'} ||  $ENV{'TMP'} || '.';
    $prefix .= '\\' unless $prefix =~ m{\\$};
    $prefix .=  "ot$$";

    for( $i = 0; ; ++$i )
    {
	$filename = "$prefix.$i.tmp";
	return $filename if( ! -e $filename );
    }
}

# -----------------------------------------------------------------------------
sub readfile
{
    my( $filename, $key, $subkey, $result ) = @_;
    my( $foundBlock ) = 0;

    open( REGFILE, "<$filename" ) or return $result;

    # To locate the entry:
    # 1. Scan for [key]
    # 2. Then scan for "subkey"=<value>.
    # 3. Abort scan if cannot find #2 within key block.

    while( <REGFILE> )
    {
	chomp;

	# Some versions of regedit spit out unprintable characters
	# between each character in the resulting file; remove those.
	s{[\x00-\x1B\x80-\xFF]}{}g;

	if( /^\[(.*)]$/ )
	{
	    my( $section ) = $1;

	    last if( $foundBlock );
	    $foundBlock = 1 if $section =~ m/\Q$key\E/i;
	}

	next unless $foundBlock;

	# inside the key block
	if( /^\"(.*)\"=\"(.*)\"$/ )
	{
	    my( $name, $value ) = ( $1, $2 );

	    if( $name =~ m/\Q$subkey\E/i )
	    {
		# Found the key
		$result = $value;
		$result =~ s/\\\\/\\/g;
		last;
	    }
	}
    }

    close( REGFILE );

    return $result;
}

# -----------------------------------------------------------------------------
sub ReadRegEntry
{
    my( $key, $subkey, $result ) = @_;
    my( $tempfile, $status );

    # Create a temporary filename for output of regedit
    $tempfile = &tempfile();

    $status = system( 'regedit', '/e', $tempfile, "\"$key\"" );

    if( $status == 0 )
    {
	# Successfully invoked regedit - now parse file
	$result = readfile( $tempfile, $key, $subkey, $result );
    }

    unlink( $tempfile );

    return $result;
}

1; # for require
