/* File dos2unix.c
 * Author: Ted Unangst <grendel@heorot.stanford.edu>
 * Converts DOS text files to UNIX format, by stripping the carriage return.  
 * If called as unix2dos, does the reverse.
 * Bug: If the last line of the DOS file doesn't have a CR, one is added.
 * However, even with multiple conversions, only one CR is ever tacked on.
 *
 * Instructions:  
 * 1. gcc -o dos2unix dos2unix.c
 * 2. ln -s dos2unix unix2dos
 */

#include <stdio.h>

void Usage();
void Error();

int main(int argc, char *argv[])
{
	char *name;
	int buffer;
	FILE *infile, *outfile;
	name = argv[0];
	
	if (argc > 1 && strcmp(argv[1], "-h") == 0)
		Usage(name);

	if (argc > 3)
		Usage(name);
	
	switch (argc) {
		case 1:
			infile = stdin;
			outfile = stdout;
			break;
		case 2:
			if ((infile = (FILE *)fopen(argv[1], "r")) 
					== NULL)
				Error(name, 1, argv[1]);
			outfile = stdout;
			break;
		case 3:
			if ((infile = (FILE *)fopen(argv[1], "r")) 
					== NULL)
				Error(name, 1, argv[1]);
			if ((outfile = (FILE *)fopen(argv[2], "w")) == NULL)
				Error(name, 2, argv[2]);
			break;
		default:
		;
	}
	
	if (strcmp(name, "unix2dos") == 0) {
		buffer = getc(infile);
		while (buffer != EOF) {
			if (buffer == 012)
				putc(015, outfile);
			putc(buffer, outfile);
			buffer = getc(infile);
		}
	}
	else {
		buffer = getc(infile);
		while (buffer != EOF) {
			if (buffer != 015)
				putc(buffer, outfile);
			buffer = getc(infile);
		}
	}

	return (0);
}


void Usage(char *name)
{
	if (strcmp(name, "dos2unix") == 0) {
		fprintf(stderr, "Usage: %s dosfile unixfile\n", name);
		exit(-1);
	}

	if (strcmp(name, "unix2dos") == 0) {
		fprintf(stderr, "Usage: %s unixfile dosfile\n", name);
		exit(-1);
	}
	
	fprintf(stderr, "Usage: %s infile outfile\n", name);
	exit(-1);



}

void Error(char *name, int error, char *problem)
{
	switch (error) {
		case 1:
			fprintf(stderr, "%s: %s could not be opened.\n", 
					name, problem);
			exit (error);
		case 2: 
			fprintf(stderr, "%s: %s could not be created.\n", 
					name, problem);
			exit (error);
		default:
			fprintf(stderr, "%s: Unknown error: %s\n", 
					name, problem);
			exit (error);
	}

}

