// {{{RME classifier 'Logical View::Basic::TrafficLight'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "TrafficLight.h"
#endif

#include <RTSystem/AustrianIntersection_x86gcc.h>
#include <TrafficLight.h>
#include <Constants.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR
// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"betweenLights"
	  , &BetweenLights::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"WalkSign"
	  , &OnOffProtocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_TrafficLight_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new TrafficLight_Actor( _rts, _ref );
}

const RTActorClass TrafficLight =
{
	(const RTActorClass *)0
  , "TrafficLight"
  , (RTVersionId)0
  , 2
  , rtg_relays
  , new_TrafficLight_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "initializing"
  , "flashingAmber"
  , "on"
  , "off"
  , "working"
  , "green"
  , "transitioning"
  , "red"
  , "IHavePriority"
};

#define SUPER RTActor

TrafficLight_Actor::TrafficLight_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, myId( 0 )
{
}

TrafficLight_Actor::~TrafficLight_Actor( void )
{
}

int TrafficLight_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// betweenLights
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &betweenLights;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// WalkSign
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &WalkSign;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Initial:Initial'
INLINE_METHODS void TrafficLight_Actor::transition1_Initial( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	myId = idProvider.get();
	timer.informIn(RTTimespec(2,0));

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Junction1:timeout'
INLINE_METHODS void TrafficLight_Actor::transition4_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	betweenLights.id(myId).send();
	//Recall any deferred messages
	betweenLights.recall();

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Junction3:idToEarly'
INLINE_METHODS void TrafficLight_Actor::transition5_idToEarly( const int * rtdata, BetweenLights::Base * rtport )
{
	// {{{USR
	getMsg()->defer();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:initializing:flashingAmber:Junction2:msgFromOtherLight'
INLINE_METHODS void TrafficLight_Actor::transition6_msgFromOtherLight( const int * rtdata, BetweenLights::Base * rtport )
{
	// {{{USR
	timer.cancelTimer(timerId);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:initializing:flashingAmber:Junction1:Initial'
INLINE_METHODS void TrafficLight_Actor::transition10_Initial( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	timerId = timer.informEvery(RTTimespec(1,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:initializing:flashingAmber:on:Junction3:msgFromOtherLight'
INLINE_METHODS void TrafficLight_Actor::transition14_msgFromOtherLight( const int * rtdata, BetweenLights::Base * rtport )
{
	// {{{USR
	//turn the amber light off
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:working:transitioning:Junction2:timeout'
INLINE_METHODS void TrafficLight_Actor::transition18_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	betweenLights.turningRed().send();

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:working:green:Junction2:True'
INLINE_METHODS void TrafficLight_Actor::transition21_True( const int * rtdata, BetweenLights::Base * rtport )
{
	// {{{USR
	timer.informIn(RTTimespec(10,0));
	// }}}USR
}
// }}}RME

INLINE_CHAINS void TrafficLight_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	transition1_Initial( msg->data, msg->sap() );
	rtgTransitionEnd();
	processHistory();
}

INLINE_CHAINS void TrafficLight_Actor::chain4_timeout( void )
{
	// transition ':TOP:Junction1:timeout'
	rtgChainBegin( 1, "timeout" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition4_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
	// transition ':TOP:initializing:Initial_0:Initial'
	rtgChainBegin( 2, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 3 );
	// transition ':TOP:initializing:flashingAmber:Junction1:Initial'
	rtgChainBegin( 3, "Initial" );
	rtgTransitionBegin();
	transition10_Initial( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void TrafficLight_Actor::chain5_idToEarly( void )
{
	// transition ':TOP:Junction3:idToEarly'
	rtgChainBegin( 1, "idToEarly" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition5_idToEarly( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	processHistory();
}

INLINE_CHAINS void TrafficLight_Actor::chain12_timeout( void )
{
	// transition ':TOP:initializing:flashingAmber:on:Junction2:timeout'
	rtgChainBegin( 4, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void TrafficLight_Actor::chain14_msgFromOtherLight( void )
{
	// transition ':TOP:initializing:flashingAmber:on:Junction3:msgFromOtherLight'
	rtgChainBegin( 4, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition14_msgFromOtherLight( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	// transition ':TOP:initializing:flashingAmber:Junction2:msgFromOtherLight'
	rtgChainBegin( 3, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_msgFromOtherLight( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint1_IHavePriority( (const int *)msg->data, (BetweenLights::Base *)msg->sap() ) )
		chain8_True();
	else
		chain9_False();
}

// {{{RME choicePoint ':TOP:initializing:IHavePriority'
INLINE_METHODS int TrafficLight_Actor::choicePoint1_IHavePriority( const int * rtdata, BetweenLights::Base * rtport )
{
	// {{{USR
	return (myId > *rtdata); 
	// }}}USR
}
// }}}RME

INLINE_CHAINS void TrafficLight_Actor::chain8_True( void )
{
	// transition ':TOP:initializing:IHavePriority:True'
	rtgChainBegin( 10, "True" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// transition ':TOP:initializing:True:IGoFirst'
	rtgChainBegin( 2, "IGoFirst" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 6 );
	// transition ':TOP:working:True:True'
	rtgChainBegin( 6, "True" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 7 );
	// transition ':TOP:working:green:Junction2:True'
	rtgChainBegin( 7, "True" );
	rtgTransitionBegin();
	transition21_True( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void TrafficLight_Actor::chain9_False( void )
{
	// transition ':TOP:initializing:IHavePriority:False'
	rtgChainBegin( 10, "False" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// transition ':TOP:initializing:Junction1:IGoLater'
	rtgChainBegin( 2, "IGoLater" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 6 );
	// transition ':TOP:working:Junction1:IGoLater'
	rtgChainBegin( 6, "IGoLater" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 9 );
}

INLINE_CHAINS void TrafficLight_Actor::chain11_timeout( void )
{
	// transition ':TOP:initializing:flashingAmber:off:Junction2:timeout'
	rtgChainBegin( 5, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void TrafficLight_Actor::chain13_msgFromOtherLight( void )
{
	// transition ':TOP:initializing:flashingAmber:off:Junction4:msgFromOtherLight'
	rtgChainBegin( 5, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// transition ':TOP:initializing:flashingAmber:Junction2:msgFromOtherLight'
	rtgChainBegin( 3, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_msgFromOtherLight( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint1_IHavePriority( (const int *)msg->data, (BetweenLights::Base *)msg->sap() ) )
		chain8_True();
	else
		chain9_False();
}

INLINE_CHAINS void TrafficLight_Actor::chain16_greenTimeout( void )
{
	// transition ':TOP:working:green:Junction1:greenTimeout'
	rtgChainBegin( 7, "greenTimeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 8 );
}

INLINE_CHAINS void TrafficLight_Actor::chain18_timeout( void )
{
	// transition ':TOP:working:transitioning:Junction2:timeout'
	rtgChainBegin( 8, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition18_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 9 );
}

INLINE_CHAINS void TrafficLight_Actor::chain17_otherRed( void )
{
	// transition ':TOP:working:red:Junction3:otherRed'
	rtgChainBegin( 9, "otherRed" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 8 );
}

void TrafficLight_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain4_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_id:
					chain5_idToEarly();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:initializing'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:initializing:flashingAmber'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:initializing:flashingAmber:on'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain12_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_id:
					chain14_msgFromOtherLight();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME state ':TOP:initializing:flashingAmber:off'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain11_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_id:
					chain13_msgFromOtherLight();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 6:
			// {{{RME state ':TOP:working'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 7:
			// {{{RME state ':TOP:working:green'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain16_greenTimeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 8:
			// {{{RME state ':TOP:working:transitioning'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain18_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 9:
			// {{{RME state ':TOP:working:red'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_turningRed:
					chain17_otherRed();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * TrafficLight_Actor::getActorData( void ) const
{
	return &TrafficLight_Actor::rtg_class;
}

const RTActor_class TrafficLight_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 9
  , TrafficLight_Actor::rtg_parent_state
  , &TrafficLight
  , 0
  , (const RTComponentDescriptor *)0
  , 3
  , TrafficLight_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 3
  , TrafficLight_Actor::rtg_TrafficLight_fields
};

const RTStateId TrafficLight_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 2
  , 3
  , 3
  , 1
  , 6
  , 6
  , 6
};

const RTPortDescriptor TrafficLight_Actor::rtg_ports[] =
{
	{
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( TrafficLight_Actor, TrafficLight_Actor::timer )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::Timing
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterAutoUnlockedUnpublished
	  , RTPortDescriptor::ProtectedVisibility
	  , RTPortDescriptor::Unwired
	}
  , {
		"betweenLights"
	  , (const char *)0
	  , &BetweenLights::Base::rt_class
	  , RTOffsetOf( TrafficLight_Actor, TrafficLight_Actor::betweenLights )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::End
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterNotPermitted
	  , RTPortDescriptor::PublicVisibility
	  , RTPortDescriptor::Wired
	}
  , {
		"WalkSign"
	  , (const char *)0
	  , &OnOffProtocol::Conjugate::rt_class
	  , RTOffsetOf( TrafficLight_Actor, TrafficLight_Actor::WalkSign )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::End
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterNotPermitted
	  , RTPortDescriptor::PublicVisibility
	  , RTPortDescriptor::Wired
	}
};

const RTFieldDescriptor TrafficLight_Actor::rtg_TrafficLight_fields[] =
{
	// {{{RME classAttribute 'timerId'
	{
		"timerId"
	  , RTOffsetOf( TrafficLight_Actor, timerId )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_RTTimerId
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'myId'
  , {
		"myId"
	  , RTOffsetOf( TrafficLight_Actor, myId )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'idProvider'
  , {
		"idProvider"
	  , RTOffsetOf( TrafficLight_Actor, idProvider )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_IdProvider
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR
// }}}USR
// }}}RME

// }}}RME
