// {{{RME classifier 'Logical View::VariantAustrian::TrafficLightAustrian'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "TrafficLightAustrian.h"
#endif

#include <RTSystem/AustrianIntersection_x86gcc.h>
#include <TrafficLightAustrian.h>
#include <Constants.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR
// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"betweenLights"
	  , &BetweenLights::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"WalkSign"
	  , &OnOffProtocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_TrafficLightAustrian_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new TrafficLightAustrian_Actor( _rts, _ref );
}

const RTActorClass TrafficLightAustrian =
{
	&TrafficLight
  , "TrafficLightAustrian"
  , (RTVersionId)0
  , 2
  , rtg_relays
  , new_TrafficLightAustrian_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "initializing"
  , "flashingAmber"
  , "on"
  , "off"
  , "working"
  , "green"
  , "transitioning"
  , "red"
  , "amber"
  , "flashingGreen"
  , "lightOn"
  , "lightOff"
  , "redAndAmber"
  , "IHavePriority"
  , "flashedEnough"
};

const RTFieldDescriptor rtg_TrafficLightAustrian_fields[] =
{
	// {{{RME classAttribute 'numberFlashes'
	{
		"numberFlashes"
	  , RTOffsetOf( TrafficLightAustrian_Actor, numberFlashes )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};

#define SUPER TrafficLight_Actor

TrafficLightAustrian_Actor::TrafficLightAustrian_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: TrafficLight_Actor( rtg_rts, rtg_ref )
	, numberFlashes( 0 )
{
}

TrafficLightAustrian_Actor::~TrafficLightAustrian_Actor( void )
{
}

// {{{RME operation 'playSound20()'
void TrafficLightAustrian_Actor::playSound20( void )
{
	// {{{USR
	char *arg_list[3];
	arg_list[0] = "/x86/usr/bin/playsound_noph";
	arg_list[1] = "/share/SOUND20.WAV";
	arg_list[2] = NULL;

	spawnvp(P_NOWAIT, "/x86/usr/bin/playsound_noph",arg_list);
	// }}}USR
}
// }}}RME

// {{{RME operation 'playSound19()'
void TrafficLightAustrian_Actor::playSound19( void )
{
	// {{{USR
	char *arg_list[3];
	arg_list[0] = "/x86/usr/bin/playsound_noph";
	arg_list[1] = "/share/SOUND19.WAV";
	arg_list[2] = NULL;

	spawnvp(P_NOWAIT, "/x86/usr/bin/playsound_noph",arg_list);
	// }}}USR
}
// }}}RME

int TrafficLightAustrian_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// betweenLights
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &betweenLights;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// WalkSign
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &WalkSign;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return TrafficLight_Actor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME enter ':TOP:working:green'
#define CALLSUPER TrafficLight_Actor::enter7_green()
INLINE_METHODS void TrafficLightAustrian_Actor::enter7_green( void )
{
	// {{{USR
	playSound19();
	// }}}USR
}
#undef CALLSUPER
// }}}RME

void TrafficLightAustrian_Actor::enterStateV( void )
{
	switch( getCurrentState() )
	{
	case 7:
		enter7_green();
		break;
	case 9:
		enter9_red();
		break;
	default:
		TrafficLight_Actor::enterStateV();
		break;
	}
}

// {{{RME enter ':TOP:working:red'
#define CALLSUPER TrafficLight_Actor::enter9_red()
INLINE_METHODS void TrafficLightAustrian_Actor::enter9_red( void )
{
	// {{{USR
	playSound20();
	if (WalkSign.isBoundAt(0)){
		WalkSign.on().send();
		};
	// }}}USR
}
#undef CALLSUPER
// }}}RME

// {{{RME exit ':TOP:working:red'
#define CALLSUPER TrafficLight_Actor::exit9_red()
INLINE_METHODS void TrafficLightAustrian_Actor::exit9_red( void )
{
	// {{{USR
	if (WalkSign.isBoundAt(0)){
		WalkSign.off().send();
		};
	// }}}USR
}
#undef CALLSUPER
// }}}RME

void TrafficLightAustrian_Actor::exitStateV( void )
{
	switch( getCurrentState() )
	{
	case 9:
		exit9_red();
		break;
	default:
		TrafficLight_Actor::exitStateV();
		break;
	}
}

// {{{RME transition ':TOP:working:green:Junction2:True'
#define SUPERMETHOD TrafficLight_Actor::transition21_True
#define CALLSUPER TrafficLight_Actor::transition21_True( rtdata, rtport )
INLINE_METHODS void TrafficLightAustrian_Actor::transition21_True( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	timer.informIn(RTTimespec(10,0));
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:working:transitioning:flashingGreen:Junction2:timeout'
#define SUPERMETHOD TrafficLight_Actor::transition23_timeout
#define CALLSUPER TrafficLight_Actor::transition23_timeout( rtdata, rtport )
INLINE_METHODS void TrafficLightAustrian_Actor::transition23_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	timer.informIn(RTTimespec(2,0));
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:working:transitioning:Junction1:timeout3'
#define SUPERMETHOD TrafficLight_Actor::transition24_timeout3
#define CALLSUPER TrafficLight_Actor::transition24_timeout3( rtdata, rtport )
INLINE_METHODS void TrafficLightAustrian_Actor::transition24_timeout3( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	timerId = timer.informEvery(RTTimespec(1,0));
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:working:transitioning:Junction3:otherRed'
#define SUPERMETHOD TrafficLight_Actor::transition25_otherRed
#define CALLSUPER TrafficLight_Actor::transition25_otherRed( rtdata, rtport )
INLINE_METHODS void TrafficLightAustrian_Actor::transition25_otherRed( const void * rtdata, BetweenLights::Base * rtport )
{
	// {{{USR
	timer.informIn(RTTimespec(2,0));
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:working:transitioning:flashingGreen:Junction1:timeout3'
#define SUPERMETHOD TrafficLight_Actor::transition27_timeout3
#define CALLSUPER TrafficLight_Actor::transition27_timeout3( rtdata, rtport )
INLINE_METHODS void TrafficLightAustrian_Actor::transition27_timeout3( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	numberFlashes = 0;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:working:transitioning:flashingGreen:lightOff:Junction2:timeout'
#define SUPERMETHOD TrafficLight_Actor::transition28_timeout
#define CALLSUPER TrafficLight_Actor::transition28_timeout( rtdata, rtport )
INLINE_METHODS void TrafficLightAustrian_Actor::transition28_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	numberFlashes++;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:working:transitioning:flashingGreen:flashedEnough:True'
#define SUPERMETHOD TrafficLight_Actor::transition29_True
#define CALLSUPER TrafficLight_Actor::transition29_True( rtdata, rtport )
INLINE_METHODS void TrafficLightAustrian_Actor::transition29_True( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	timer.cancelTimer(timerId);
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

INLINE_CHAINS void TrafficLightAustrian_Actor::chain1_Initial( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	transition1_Initial( msg->data, msg->sap() );
	rtgTransitionEnd();
	processHistory();
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain4_timeout( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:Junction1:timeout'
	rtgChainBegin( 1, "timeout" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition4_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:Initial_0:Initial'
	rtgChainBegin( 2, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 3 );
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:flashingAmber:Junction1:Initial'
	rtgChainBegin( 3, "Initial" );
	rtgTransitionBegin();
	transition10_Initial( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain5_idToEarly( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:Junction3:idToEarly'
	rtgChainBegin( 1, "idToEarly" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition5_idToEarly( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	processHistory();
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain12_timeout( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:flashingAmber:on:Junction2:timeout'
	rtgChainBegin( 4, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain14_msgFromOtherLight( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:flashingAmber:on:Junction3:msgFromOtherLight'
	rtgChainBegin( 4, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition14_msgFromOtherLight( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:flashingAmber:Junction2:msgFromOtherLight'
	rtgChainBegin( 3, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_msgFromOtherLight( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint1_IHavePriority( (const int *)msg->data, (BetweenLights::Base *)msg->sap() ) )
		chain8_True();
	else
		chain9_False();
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain8_True( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:IHavePriority:True'
	rtgChainBegin( 15, "True" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:True:IGoFirst'
	rtgChainBegin( 2, "IGoFirst" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 6 );
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:working:True:True'
	rtgChainBegin( 6, "True" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 7 );
	// transition ':TOP:working:green:Junction2:True'
	rtgChainBegin( 7, "True" );
	rtgTransitionBegin();
	transition21_True( msg->data, msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain9_False( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:IHavePriority:False'
	rtgChainBegin( 15, "False" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:Junction1:IGoLater'
	rtgChainBegin( 2, "IGoLater" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 6 );
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:working:Junction1:IGoLater'
	rtgChainBegin( 6, "IGoLater" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 9 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain11_timeout( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:flashingAmber:off:Junction2:timeout'
	rtgChainBegin( 5, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain13_msgFromOtherLight( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:flashingAmber:off:Junction4:msgFromOtherLight'
	rtgChainBegin( 5, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:initializing:flashingAmber:Junction2:msgFromOtherLight'
	rtgChainBegin( 3, "msgFromOtherLight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_msgFromOtherLight( (const int *)msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint1_IHavePriority( (const int *)msg->data, (BetweenLights::Base *)msg->sap() ) )
		chain8_True();
	else
		chain9_False();
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain16_greenTimeout( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:working:green:Junction1:greenTimeout'
	rtgChainBegin( 7, "greenTimeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 8 );
	// transition ':TOP:working:transitioning:Junction1:timeout3'
	rtgChainBegin( 8, "timeout3" );
	rtgTransitionBegin();
	transition24_timeout3( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 11 );
	// transition ':TOP:working:transitioning:flashingGreen:Junction1:timeout3'
	rtgChainBegin( 11, "timeout3" );
	rtgTransitionBegin();
	transition27_timeout3( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 13 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain17_otherRed( void )
{
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:working:red:Junction3:otherRed'
	rtgChainBegin( 9, "otherRed" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 8 );
	// transition ':TOP:working:transitioning:Junction3:otherRed'
	rtgChainBegin( 8, "otherRed" );
	rtgTransitionBegin();
	transition25_otherRed( msg->data, (BetweenLights::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 14 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain22_timeout( void )
{
	// transition ':TOP:working:transitioning:amber:Junction2:timeout'
	rtgChainBegin( 10, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:working:transitioning:Junction2:timeout'
	rtgChainBegin( 8, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition18_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 9 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain31_timeout( void )
{
	// transition ':TOP:working:transitioning:flashingGreen:lightOn:Junction2:timeout'
	rtgChainBegin( 12, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 13 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain28_timeout( void )
{
	// transition ':TOP:working:transitioning:flashingGreen:lightOff:Junction2:timeout'
	rtgChainBegin( 13, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition28_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint2_flashedEnough( msg->data, (Timing::Base *)msg->sap() ) )
		chain29_True();
	else
		chain30_False();
}

// {{{RME choicePoint ':TOP:working:transitioning:flashingGreen:flashedEnough'
#define SUPERMETHOD TrafficLight_Actor::choicePoint2_flashedEnough
#define CALLSUPER TrafficLight_Actor::choicePoint2_flashedEnough( rtdata, rtport )
INLINE_METHODS int TrafficLightAustrian_Actor::choicePoint2_flashedEnough( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	return (numberFlashes > GreenFlashes);
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

INLINE_CHAINS void TrafficLightAustrian_Actor::chain29_True( void )
{
	// transition ':TOP:working:transitioning:flashingGreen:flashedEnough:True'
	rtgChainBegin( 16, "True" );
	rtgTransitionBegin();
	transition29_True( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	// transition ':TOP:working:transitioning:flashingGreen:Junction2:timeout'
	rtgChainBegin( 11, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition23_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 10 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain30_False( void )
{
	// transition ':TOP:working:transitioning:flashingGreen:flashedEnough:False'
	rtgChainBegin( 16, "False" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 12 );
}

INLINE_CHAINS void TrafficLightAustrian_Actor::chain26_timeout( void )
{
	// transition ':TOP:working:transitioning:redAndAmber:Junction1:timeout'
	rtgChainBegin( 14, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	// classifier 'Logical View::Basic::TrafficLight' transition ':TOP:working:transitioning:Junction4:timeout2'
	rtgChainBegin( 8, "timeout2" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 7 );
	// transition ':TOP:working:green:Junction2:True'
	rtgChainBegin( 7, "True" );
	rtgTransitionBegin();
	transition21_True( msg->data, msg->sap() );
	rtgTransitionEnd();
}

void TrafficLightAustrian_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain4_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_id:
					chain5_idToEarly();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME classifier 'Logical View::Basic::TrafficLight' state ':TOP:initializing'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME classifier 'Logical View::Basic::TrafficLight' state ':TOP:initializing:flashingAmber'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME classifier 'Logical View::Basic::TrafficLight' state ':TOP:initializing:flashingAmber:on'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain12_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_id:
					chain14_msgFromOtherLight();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME classifier 'Logical View::Basic::TrafficLight' state ':TOP:initializing:flashingAmber:off'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain11_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_id:
					chain13_msgFromOtherLight();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 6:
			// {{{RME state ':TOP:working'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 7:
			// {{{RME state ':TOP:working:green'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain16_greenTimeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 8:
			// {{{RME state ':TOP:working:transitioning'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 9:
			// {{{RME state ':TOP:working:red'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'betweenLights'
				switch( signalIndex )
				{
				case BetweenLights::Base::rti_turningRed:
					chain17_otherRed();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 10:
			// {{{RME state ':TOP:working:transitioning:amber'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain22_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 11:
			// {{{RME state ':TOP:working:transitioning:flashingGreen'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 12:
			// {{{RME state ':TOP:working:transitioning:flashingGreen:lightOn'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain31_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 13:
			// {{{RME state ':TOP:working:transitioning:flashingGreen:lightOff'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain28_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 14:
			// {{{RME state ':TOP:working:transitioning:redAndAmber'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::Basic::TrafficLight' port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain26_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * TrafficLightAustrian_Actor::getActorData( void ) const
{
	return &TrafficLightAustrian_Actor::rtg_class;
}

const RTActor_class TrafficLightAustrian_Actor::rtg_class =
{
	&TrafficLight_Actor::rtg_class
  , rtg_state_names
  , 14
  , TrafficLightAustrian_Actor::rtg_parent_state
  , &TrafficLightAustrian
  , 0
  , (const RTComponentDescriptor *)0
  , 3
  , TrafficLightAustrian_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , rtg_TrafficLightAustrian_fields
};

const RTStateId TrafficLightAustrian_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 2
  , 3
  , 3
  , 1
  , 6
  , 6
  , 6
  , 8
  , 8
  , 11
  , 11
  , 8
};

const RTPortDescriptor TrafficLightAustrian_Actor::rtg_ports[] =
{
	{
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( TrafficLightAustrian_Actor, TrafficLight_Actor::timer )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::Timing
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterAutoUnlockedUnpublished
	  , RTPortDescriptor::ProtectedVisibility
	  , RTPortDescriptor::Unwired
	}
  , {
		"betweenLights"
	  , (const char *)0
	  , &BetweenLights::Base::rt_class
	  , RTOffsetOf( TrafficLightAustrian_Actor, TrafficLight_Actor::betweenLights )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::End
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterNotPermitted
	  , RTPortDescriptor::PublicVisibility
	  , RTPortDescriptor::Wired
	}
  , {
		"WalkSign"
	  , (const char *)0
	  , &OnOffProtocol::Conjugate::rt_class
	  , RTOffsetOf( TrafficLightAustrian_Actor, TrafficLight_Actor::WalkSign )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::End
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterNotPermitted
	  , RTPortDescriptor::PublicVisibility
	  , RTPortDescriptor::Wired
	}
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR
// }}}USR
// }}}RME

// }}}RME
