// {{{RME classifier 'Logical View::Hardware::WalkLight'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "WalkLight.h"
#endif

#include <RTSystem/AustrianIntersection_x86gcc.h>
#include <WalkLight.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR
// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"switchPort"
	  , &OnOffProtocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_WalkLight_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new WalkLight_Actor( _rts, _ref );
}

const RTActorClass WalkLight =
{
	(const RTActorClass *)0
  , "WalkLight"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_WalkLight_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Off"
  , "On"
};

#define SUPER RTActor

WalkLight_Actor::WalkLight_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

WalkLight_Actor::~WalkLight_Actor( void )
{
}

// {{{RME operation 'setAllPixels()'
void WalkLight_Actor::setAllPixels( void )
{
	// {{{USR
	log.log("  #    #    ##    #       #    #");
	log.log("  #    #   #  #   #       #   #");
	log.log("  #    #  #    #  #       ####");
	log.log("  # ## #  ######  #       #  #");
	log.log("  ##  ##  #    #  #       #   #");
	log.log("  #    #  #    #  ######  #    #");
	playCrossSound();
	// }}}USR
}
// }}}RME

// {{{RME operation 'unsetAllPixels()'
void WalkLight_Actor::unsetAllPixels( void )
{
	// {{{USR
	for (int i = 0; i < 100; i++)
	{
		log.log(" ");
	}
	// }}}USR
}
// }}}RME

// {{{RME operation 'playCrossSound()'
void WalkLight_Actor::playCrossSound( void )
{
	// {{{USR
	char *arg_list[3];
	arg_list[0] = "/x86/usr/bin/playsound_noph";
	arg_list[1] = "/share/ALERTK2.wav";
	arg_list[2] = NULL;

	spawnvp(P_NOWAIT, "/x86/usr/bin/playsound_noph",arg_list);
	// }}}USR
}
// }}}RME

int WalkLight_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// switchPort
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &switchPort;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Initial:Initial'
INLINE_METHODS void WalkLight_Actor::transition1_Initial( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	unsetAllPixels();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Off:Junction2:on'
INLINE_METHODS void WalkLight_Actor::transition2_on( const void * rtdata, OnOffProtocol::Base * rtport )
{
	// {{{USR
	setAllPixels();
	playCrossSound();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:On:Junction2:off'
INLINE_METHODS void WalkLight_Actor::transition3_off( const void * rtdata, OnOffProtocol::Base * rtport )
{
	// {{{USR
	unsetAllPixels();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Junction1:test'
INLINE_METHODS void WalkLight_Actor::transition4_test( const void * rtdata, OnOffProtocol::Base * rtport )
{
	// {{{USR
	playCrossSound();
	// }}}USR
}
// }}}RME

INLINE_CHAINS void WalkLight_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	transition1_Initial( msg->data, msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void WalkLight_Actor::chain4_test( void )
{
	// transition ':TOP:Junction1:test'
	rtgChainBegin( 1, "test" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition4_test( msg->data, (OnOffProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	processHistory();
}

INLINE_CHAINS void WalkLight_Actor::chain2_on( void )
{
	// transition ':TOP:Off:Junction2:on'
	rtgChainBegin( 2, "on" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_on( msg->data, (OnOffProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void WalkLight_Actor::chain3_off( void )
{
	// transition ':TOP:On:Junction2:off'
	rtgChainBegin( 3, "off" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_off( msg->data, (OnOffProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void WalkLight_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'switchPort'
				switch( signalIndex )
				{
				case OnOffProtocol::Base::rti_testSound:
					chain4_test();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Off'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'switchPort'
				switch( signalIndex )
				{
				case OnOffProtocol::Base::rti_on:
					chain2_on();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:On'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'switchPort'
				switch( signalIndex )
				{
				case OnOffProtocol::Base::rti_off:
					chain3_off();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * WalkLight_Actor::getActorData( void ) const
{
	return &WalkLight_Actor::rtg_class;
}

const RTActor_class WalkLight_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 3
  , WalkLight_Actor::rtg_parent_state
  , &WalkLight
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , WalkLight_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId WalkLight_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
};

const RTPortDescriptor WalkLight_Actor::rtg_ports[] =
{
	{
		"switchPort"
	  , (const char *)0
	  , &OnOffProtocol::Base::rt_class
	  , RTOffsetOf( WalkLight_Actor, WalkLight_Actor::switchPort )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::End
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterNotPermitted
	  , RTPortDescriptor::PublicVisibility
	  , RTPortDescriptor::Wired
	}
  , {
		"log"
	  , (const char *)0
	  , &Log::Base::rt_class
	  , RTOffsetOf( WalkLight_Actor, WalkLight_Actor::log )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::Log
	  , RTPortDescriptor::NotificationDisabled
	  , RTPortDescriptor::RegisterAutoUnlockedUnpublished
	  , RTPortDescriptor::ProtectedVisibility
	  , RTPortDescriptor::Unwired
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR
// }}}USR
// }}}RME

// }}}RME
